﻿/* ============================================================
* Author:       liubingsheng
* Time:         2016/11/29 14:52:25
* FileName:     RedisSystem
* Purpose:      Redis
* =============================================================*/

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Security.Cryptography;
using System.Text;

namespace Utility
{
    /// <summary>
    /// String扩展
    /// </summary>
    public static class StringExtension
    {
        /// <summary>
        /// 是否包含
        /// </summary>
        /// <param name="self"></param>
        /// <param name="tocheck"></param>
        /// <returns></returns>
        public static bool ContainsIgnoreCase(this string self, string tocheck)
        {
            bool contains = self.IndexOf(tocheck, StringComparison.OrdinalIgnoreCase) >= 0;

            return contains;
        }

        /// <summary>
        /// 是否相等
        /// </summary>
        /// <param name="self"></param>
        /// <param name="tocheck"></param>
        /// <returns></returns>
        public static bool IsEqualIgnoreCase(this string self, string tocheck)
        {
            return self.ToLower() == tocheck.ToLower();
        }

        /// <summary>
        /// 是否是合法的年月日格式
        /// </summary>
        /// <param name="self"></param>
        /// <param name="check"></param>
        /// <returns></returns>
        public static bool IsValidYMD(this string self)
        {
            DateTime dateTime;
            if (DateTime.TryParseExact(self, "yyyyMMdd", CultureInfo.CurrentCulture, DateTimeStyles.AllowInnerWhite, out dateTime))
            {
                return true;
            }
            return false;
        }

        public static DateTime ToYmdDatetime(this string self)
        {
            DateTime dateTime;
            if (DateTime.TryParseExact(self, "yyyyMMdd", CultureInfo.CurrentCulture, DateTimeStyles.AllowInnerWhite, out dateTime))
            {
                return dateTime;
            }
            return DateTime.MinValue;
        }

        /// <summary>
        /// 转化为日
        /// </summary>
        /// <param name="self"></param>
        /// <returns></returns>
        public static Date ToDate(this string self)
        {
            DateTime dateTime;
            if (DateTime.TryParseExact(self, "yyyyMMdd", CultureInfo.CurrentCulture, DateTimeStyles.AllowInnerWhite, out dateTime))
            {
                return new Date(dateTime);
            }
            return Date.MinValue;
        }

        /// <summary>
        /// 转化为周
        /// </summary>
        /// <param name="self"></param>
        /// <returns></returns>
        public static Week ToWeek(this string self)
        {
            DateTime dateTime;
            if (DateTime.TryParseExact(self, "yyyyMMdd", CultureInfo.CurrentCulture, DateTimeStyles.AllowInnerWhite, out dateTime))
            {
                return new Week(dateTime);
            }
            return Week.MinValue;
        }

        /// <summary>
        /// 转化为月
        /// </summary>
        /// <param name="self"></param>
        /// <returns></returns>
        public static Month ToMonth(this string self)
        {
            DateTime dateTime;
            if (DateTime.TryParseExact(self, "yyyyMMdd", CultureInfo.CurrentCulture, DateTimeStyles.AllowInnerWhite, out dateTime))
            {
                return new Month(dateTime);
            }
            return Month.MinValue;
        }


        /// <summary>
        /// 去掉空格后是否为空
        /// </summary>
        /// <param name="self"></param>
        /// <returns></returns>
        public static bool IsDateEmpty(this string self)
        {
            self.Trim();
            return string.IsNullOrEmpty(self);
        }

        public static string MergeKey<T>(this string table, T value)
        {
            string key = table + "_" + value.ToString();
            return key;
        }

        public static List<string> MergeKey<T>(this string table, List<T> lsValues)
        {
            List<string> lstResults = new List<string>();
            foreach (var value in lsValues)
            {
                string dbkey = table.MergeKey(value);
                lstResults.Add(dbkey);
            }
            return lstResults;
        }
        /// <summary>
        /// 去除所有空格
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        public static string TrimAll(this string value)
        {
            return value.Replace(" ", "");
        }

        /// <summary>
        /// 转为int
        /// </summary>
        /// <param name=""></param>
        /// <returns></returns>
        public static int ToInt(this string input)
        {
            int result = 0;
            if (int.TryParse(input, out result))
            {
                return result;
            }
            return result;
        }

        public static UInt32 ToUInt32(this string input)
        {
            UInt32 result = 0;
            if (UInt32.TryParse(input, out result))
            {
                return result;
            }
            return result;
        }

        public static UInt64 ToUInt64(this string input)
        {
            UInt64 result = 0;
            if (UInt64.TryParse(input, out result))
            {
                return result;
            }
            return result;
        }

        public static double ToDouble(this string input)
        {
            double result = 0;
            if (double.TryParse(input, out result))
            {
                return result;
            }
            return result;
        }

        public static float ToFloat(this string input)
        {
            float result = 0;
            if (float.TryParse(input, out result))
                return result;
            return result;
        }

        public static UInt64 ToUint64(this string input)
        {
            UInt64 result = 0;
            if (UInt64.TryParse(input, out result))
            {
                return result;
            }
            return result;
        }

        public static long ToLong(this string input)
        {
            long result = 0;
            if (long.TryParse(input, out result))
            {
                return result;
            }
            return result;
        }

        public static string ToMD5String(this string oriString)
        {
            MD5 md5 = new MD5CryptoServiceProvider();
            byte[] fromData = Encoding.UTF8.GetBytes(oriString);
            byte[] targetData = md5.ComputeHash(fromData);
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < targetData.Length; i++)
            {
                sb.Append(targetData[i].ToString("X2"));
            }
            return sb.ToString();
        }

        public static bool IsNullOrEmpty(this string input)
        {
            return string.IsNullOrEmpty(input);
        }

        /// <summary>
        /// 是否是合法的以太坊地址
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public static bool IsValidEthereumAddress(this string input)
        {
            if (input.IsNullOrEmpty())
            {
                return false;
            }

            if (string.IsNullOrWhiteSpace(input))
            {
                return false;
            }

            if (!input.StartsWith("0x"))
            {
                return false;
            }

            if (input.Length != 42)
            {
                return false;
            }

            System.Text.RegularExpressions.Regex reg = new System.Text.RegularExpressions.Regex(@"^[A-Za-z0-9]+$");
            if (!reg.IsMatch(input))
            {
                return false;
            }

            return true;
        }
    }
}
